<?php
declare(strict_types=1);

namespace Drupal\lms_h5p\Hook;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Render\Element;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Hook\Attribute\Preprocess;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Lms\Bundle\Course;
use Drupal\lms_xapi\XapiService;

/**
 * Hooks for LMS H5P.
 */
final class H5PHooks {

  public function __construct(
    #[Autowire(service: XapiService::class, lazy: TRUE)]
    protected XapiService $xapiService,
    #[Autowire(service: 'route_match', lazy: TRUE)]
    protected RouteMatchInterface $routeMatch,
    #[Autowire(service: 'current_user', lazy: TRUE)]
    protected AccountInterface $currentUser,
  ) {
  }

  #[Preprocess('field')]
  public function preprocessField(array &$variables): void {
    // Using preprocess avoids completely replacing the h5p_default formatter.
    if ($variables['element']['#formatter'] === 'h5p_default' && $variables['element']['#entity_type'] === 'lms_activity') {
      $entity = $variables['element']['#object'];
      $cacheable_metadata = new CacheableMetadata();
      $lms_id = $this->xapiService->getLmsId($entity, $this->currentUser, $cacheable_metadata);
      $lrs_uuid = $this->xapiService->getLrsUuid($lms_id);

      $cache_tags = $cacheable_metadata->getCacheTags();
      if (!\array_key_exists('#cache', $variables)) {
        $variables['#cache']['tags'] = $cacheable_metadata->getCacheTags();
        $variables['#cache']['context'] = $cacheable_metadata->getCacheContexts();
      }
      else {
        $variables['#cache']['tags'] = array_merge($variables['#cache']['tags'] ?? [], $cacheable_metadata->getTags());
        $variables['#cache']['context'] = array_merge($variables['#cache']['context'] ?? [], $cacheable_metadata->getContexts());
      }
      if ($cache_tags) {
        $variables['#cache']['tags'] = $cache_tags;
      }
      $cache_contexts = $cacheable_metadata->getCacheContexts();
      if  ($cache_contexts) {
        $variables['#cache']['contexts'] = $cache_contexts;
      }
      $lrs = $this->xapiService->getRemoteLrs();
      $variables['#attached']['library'][] = 'lms_h5p/xapi';
      $variables['#attached']['drupalSettings']['lms_h5p'] = [
        'lrs_endpoint' => $lrs->getEndpoint() . 'statements',
        // This has to be here so it can be used in the LRS request.
        // @todo: consider putting this behind a 'Send requests to LRS'
        // permission, for now assumes that H5P activities are only shown to
        // users who should be able to submit LRS requests.
        'lrs_auth' => $lrs->getAuth(),
        'lrs_registration' => $lrs_uuid,
      ];
    }
  }

  #[Hook('h5p_formatter_alter')]
  public function h5pFormatterAlter(&$element, $items) {

    $cacheable_metadata = CacheableMetadata::createFromRenderArray($element);
    $entity = $items->getEntity();
    $lms_id = $this->xapiService->getLmsId($entity, $this->currentUser, $cacheable_metadata);
    $cacheable_metadata->applyTo($element);

    foreach (Element::children($element) as $key) {
      foreach ($element[$key]['#attached']['drupalSettings']['h5p']['H5PIntegration']['contents'] as &$content) {
        // Url turns into ID before it's sent to the LRS.
        $content['url'] = $lms_id;
      }
    }
  }


}
